/************************** SunDst ***************************
 * by Ewald Kendziorra sr, 2016 www.kendziorra.nl            * 
 * Before you start: read the SunDst information in "info.h" *
 *************************************************************/

#include "SunDst.h"
#include "dst.h"

void calcDst() {
  calcDst(now());
}

void calcDst(time_t t) {
  uint16_t y = year(t);
  /** fixed date? **/
  if (startMonthDay) {
    /** Month Day > 0: it's a fixed DST **/
    dstStart = calcTimeStamp(startHour, startMinute, 0, startMonthDay, startMonth, y);
    dstEnd = calcTimeStamp(endHour, endMinute, 0, endMonthDay, endMonth, y);
  }
  /** flex date DST **/
  /** last week 4/5 of the month? **/
  else if (startWeek == -1) {
    /** start **/
    dstStart = calcTimeStamp(startHour, startMinute, 0, 1, startMonth + 1, y);  // get the first day of the next mondth
    dstStart -= oneDay;  // set last day of the month
    while (startWeekDay != weekday(dstStart)) dstStart -= oneDay;
    /** end **/
    dstEnd = calcTimeStamp(endHour, endMinute, 0, 1, endMonth + 1, y);  // get the first day of the next mondth
    dstEnd -= oneDay;  // set last day of the month
    while (endWeekDay != weekday(dstEnd)) dstEnd -= oneDay;
  }
  /** week 1-4 **/
  else {
    /** start **/
    dstStart = calcTimeStamp(startHour, startMinute, 0, 1, startMonth, y);  // get the first day of the next mondth
    dstStart += (startWeek - 1) * oneWeek;  // set the week of the month
    while (startWeekDay != weekday(dstStart)) dstStart += oneDay;
    /** end **/
    dstEnd = calcTimeStamp(endHour, endMinute, 0, 1, endMonth, y);  // get the first day of the next mondth
    dstEnd += (endWeek - 1) * oneWeek;  // set the week of the month
    while (endWeekDay != weekday(dstEnd)) dstEnd += oneDay;
  }
  isDST = false;
  /** location north or south of the equator? **/
  if (lat >= 0) {
    if (t >= dstStart && t < dstEnd)   isDST = true;  // north
  }
  else if (!(t < dstStart && t >= dstEnd)) isDST = true;  // south
}

time_t calcTimeStamp(uint8_t h, uint8_t m, uint8_t s, uint8_t D, uint8_t M, uint16_t Y) {  // get UNIX time_t Time Stamp from hh:mm:ss DD-MM-YY
  // UNIX timestamp = number of seconds since 1 Jan 00:00:00 1970
  if (Y < 100) Y += 2000;
  time_t t = 0;
  for (uint16_t i = 1970; i < Y; i++) t += yearDay(31, 12, i) * oneDay;
  return t + (yearDay(D, M, Y) - 1) * oneDay + (uint32_t)h * oneHour + (uint32_t)m * 60 + s;
}

uint16_t yearDay(time_t t) {  // year days from timestamp
  return yearDay(day(t), month(t), year(t));
}

uint16_t yearDay(uint8_t d, uint8_t m, uint16_t y) {  // year days from date numbers
  int monthdays[13] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
  return monthdays[m - 1] + (monthdays[m - 1] > 31 && leapYear(y) ? 1 : 0) + d;
}

boolean leapYear(uint16_t y) {  // leap year?
  uint16_t Y = y < 100 ? y + 2000 : y;
  return ((Y % 4 == 0 && Y % 100 != 0) || Y % 400 == 0);
}

